'''
Created on 2022-18-11

@author: wf
'''
from tests.basetest import Basetest
from skg.searchengine import InternetSearch
from skg.dblp import Dblp
import pprint
from collections import Counter

class TestSearchEngine(Basetest):
    """
    test search engine
    """
    
    def setUp(self, debug=False, profile=True):
        Basetest.setUp(self, debug=debug, profile=profile)
        self.counters={
            "sites": Counter(),
            "types": Counter(),
        }
    
    def test_searchengines(self):
        """
        test search engines
        """
        i_search=InternetSearch()
        search_terms=["LinkClus: efficient clustering via heterogeneous semantic links"]
        for search_term in search_terms:
            for engine_name,results in i_search.search(search_term):
                print (engine_name)
                pprint.pprint(results)
                
    def analyse_isearch_result(self,results):
        """
        analyse the given search engine results
        """
        for record in results:
            titles=record["titles"]
            parts=titles.split("›")
            site=parts[0]
            self.counters["sites"][site]+=1
            if len(parts)>1:
                rtype=parts[1]
                self.counters["types"][rtype]+=1
            pass
        pass
    
    def check_search(self,search_term:str):
        i_search=InternetSearch()
        for engine_name,results in i_search.search(search_term):
            print (engine_name)
            self.analyse_isearch_result(results)
            pprint.pprint(results)
        for name,counter in self.counters.items():
            print(name)
            print(counter.most_common(10))
        
    def test_dblp_titles(self):
        """
        test DBLP titles
        """
        dblp=Dblp()
        limit=5
        paper_records=dblp.get_paper_records("ARCS","publishedin",limit=limit)
        for paper_record in paper_records:
            print(paper_record)
            title=paper_record["title"]
            self.check_search(title)
            
    def test_google_scholar(self):
        """
        """
        scholars=["Suchetha N. Kunnath"]
        for scholar in scholars:
            self.check_search(scholar)
        
            
    def test_paper_search(self):
        """
        test the search for papers
        """
        titles=["Constance: an intelligent data lake system"]
        for title in titles:
            self.check_search(title)
        
        
            



'''
Created on 2022-11-16

@author: wf
'''
from tests.base_skg_test import BaseSkgTest
from skg.scholar import Scholar
from skg.graph import Node
from skg.smw import SemWiki

class TestScholar(BaseSkgTest):
    """
    test concerning the scholar/author concept
    """
    
    def test_scholar_by_wikidata_id(self):
        """
        test searching a scholar/author by ORCID,dblpId or wikiDataId from
        wikidata
        """
        author_concept=self.skg_def.concepts["Scholar"]
        id_examples=[
            {
                "id_name": "orcid",
                "id_value": "0000-0003-1279-3709",
                "concept": author_concept
            },
            {
                "id_name": "dblpId",
                "id_value": "b/TimBernersLee",
                "concept": author_concept
            },
            {
                "id_name": "wikiDataId",
                "id_value": "Q80",
                "concept": author_concept
            },           
        ]
        
        def checkItem(scholar:Scholar,id_name:str,id_value:str,debug:bool=False):
            """
            check the given item
            
            Args:
                item(Node): the item to check
                id_name(str): the name of the id used to retrieve the item
                id_value(str) the value that has been used to retriebe the item
                debug(bool): if True show debug information
            """
            self.assertEqual("Tim Berners-Lee",scholar.label)
            self.assertEqual("https://scholia.toolforge.org/author/Q80",scholar.scholia_url())
            
        debug=self.debug
        debug=True
        self.check_id_examples(id_examples, createFunc=Node.from_wikidata_via_id,checkItem=checkItem,debug=debug)            

    def test_smw_markup(self):
        """
        test Semantic MediaWiki markup for a scholar
        """
        orcids=["0000-0002-4030-0978"]
        author_concept=self.skg_def.concepts["Scholar"]
        for orcid in orcids:
            scholars=Node.from_wikidata_via_id(author_concept, "orcid", orcid)
            scholar=scholars[0]
            markup=SemWiki.asMarkup(scholar)
            print (markup)
            


'''
Created on 2022-11-16

@author: wf
'''
import skg.graph

class Event(skg.graph.Node):
    """
    an instance of a scientific event
    """
    
    @classmethod
    def getSamples(cls):
        samples=[
            {
                "wikiDataId":"Q112055391",
                "title": "The Third Wikidata Workshop",
                "location": "Hangzhou",
                "point_in_time": "2022-10-24",
                "official_website": "https://wikidataworkshop.github.io/2022/"
            }
        ]
        return samples
    
class EventSeries(skg.graph.Node):
    """
    an instance of an academic event series
    """
    
    @classmethod
    def getSamples(cls):
        samples=[
            {
                "wikiDataId":"Q6053150",
                "short_name":"ISWC"
            },
            {
                "wikiDataId": "Q105491257",
                "short_name": "ECDL",
                "title": "European Conference on Research and Advanced Technology for Digital Libraries (English)",
                "official_website": "http://ecdlconference.isti.cnr.it/"
            },
            {
                "wikiDataId": "Q105695678",
                "short_name": "VNC (English)",
                "DBLP_venue_ID": "conf/vnc",
                "VIAF_ID": "267408611",
                "title": "IEEE Vehicular Networking Conference"
            },
            {
                "wikiDataId":"Q17012957",
                "short_name":"ESWC",
                "inception": "2004",
                "gndId": "1091749205"
            }
        ]
        return samples
    
class Proceedings(skg.graph.Node):
    """
    Proceedings of an event
    """
    @classmethod
    def getSamples(cls):
        samples=[
            {
                "wikiDataId": "Q115053286",
                "short_name": "Wikidata 2022 (English)",
                "title": "Proceedings of the 3rd Wikidata Workshop 2022 (English)",
                "publication_date": "2022-11-03",
                "full_work_available_at_URL": "http://ceur-ws.org/Vol-3262/"
            }
        ]
        return samples
                



'''
Created on 2022-11-22

@author: wf
'''
import re
class DOI:
    """
    Digital Object Identifier handling
    
    see e.g. https://www.wikidata.org/wiki/Property:P356
    """
    pattern=re.compile(r"(10\.[0-9]{4,}(?:\.[0-9]+)*(?:\/|%2F)(?:(?![\"&\'])\S)+)")
  
    def __init__(self):
        """
        a DOI
        """
        
    @classmethod
    def isDOI(cls,doi:str):
        """
        check that the given string is a doi
        
        Args:
            doi(str): the potential DOI string
        """
        if not doi:
            return False
        if isinstance(doi,list):
            ok=len(doi)>0
            for single_doi in doi:
                ok=ok and cls.isDOI(single_doi)
            return ok
        if not isinstance(doi,str):
            return False
        match=re.match(cls.pattern,doi)
        return bool(match)


'''
Created on 2022-11-21

@author: wf
'''
import skg.graph

class Country(skg.graph.Node):
    """
    an instance of a country
    """
    
    @classmethod
    def getSamples(cls):
        samples=[
            {
                "wikiDataId":"Q334",
                "name": "Singapore",
                "iso_code": "SG", 
                "homepage": "https://www.gov.sg/",
                "population": 5866139,
                "coordinate_location": "1°18'N, 103°48'E"
            }
        ]
        return samples



'''
Created on 2022-11-18

@author: wf
'''    
import html
import os
from jpcore.compat import Compatibility;Compatibility(0,11,1)
from jpcore.justpy_config import JpConfig
script_dir = os.path.dirname(os.path.abspath(__file__))
static_dir = script_dir+"/resources/static"
JpConfig.set("STATIC_DIRECTORY",static_dir)
# shut up justpy
JpConfig.set("VERBOSE","False")
JpConfig.setup()
from jpwidgets.bt5widgets import App,Link,About
from urllib import parse
from skg.search import SearchOptions
from skg.orcid import ORCID
from wikibot.wikiuser import WikiUser

class SkgBrowser(App):
    """
    scholary knowledge graph browser
    """
  
    def __init__(self,version,sotsog,options:SearchOptions):
        '''
        Constructor
        
        Args:
            version(Version): the version info for the app
        '''
        self.sotsog=sotsog
        import justpy as jp
        self.jp=jp
        App.__init__(self, version)
        self.addMenuLink(text='Home',icon='home', href="/")
        self.addMenuLink(text='github',icon='github', href=version.cm_url)
        self.addMenuLink(text='Chat',icon='chat',href=version.chat_url)
        self.addMenuLink(text='Documentation',icon='file-document',href=version.doc_url)
        self.addMenuLink(text='Settings',icon='cog',href="/settings")
        self.addMenuLink(text='About',icon='information',href="/about")
        self.options=options
        self.markup_names=["-","bibtex","scite","smw"]
        self.markup_name=self.markup_names[1]
        # wiki users
        self.wikiUsers=WikiUser.getWikiUsers()
        self.wikiId="ceur-ws"
        jp.Route('/settings',self.settings)
        jp.Route('/about',self.about)
        
    def createItemLink(self,item,term:str,index:int)->str:
        """
        create a link for the given item
        
        Args:
            item(Node): the item to create a link for
            term(str): the
        """
        if index>0:
            style="color:grey"
            text=f"{term}<sub>{index+1}</sub>"
            delim="&nbsp"
        else:
            style=""
            text=term
            delim=""
        link=Link.create(item.browser_url(),text,tooltip=item.label,target="_blank",style=style)
        if item.concept.name=="Scholar":
            if hasattr(item,"orcid"):
                orcid=ORCID(item.orcid)
                link+=orcid.asHtml()
        markup=delim+link
        return markup      
        
    async def onSearchButton(self,_msg):
        """
        handle button to search for terms
        """
        try:
            self.results.inner_html=""
            self.markup.inner_html=""
            terms=self.searchTerms.value.split("\n")
            self.messages.text="Searching"
            await self.wp.update()
            delim=""
            for term in terms:
                if term:
                    msg=f"... {term}\n"
                    self.messages.text+=msg
                    await self.wp.update()
                    if self.markup_name=="-":
                        self.options.markup_names=[]
                    else:
                        self.options.markup_names=[self.markup_name]
                    search_result=self.sotsog.search([term],self.options)
                    items=search_result.items
                    rmarkup=""
                    if len(items)==0:
                        # TODO check google search
                        # https://pypi.org/project/googlesearch-python/
                        params=parse.urlencode({'q':term})
                        search_url=f"https://www.google.com/search?{params}"
                        rmarkup=Link.create(search_url, term, "not found", target="_blank",style="color:red")
                    else:
                        for i,item in enumerate(items):
                            rmarkup+=self.createItemLink(item,term,i)
                            if len(item.markups)>0:
                                markups=""
                                for _markup_name,markup in item.markups.items():
                                    markups+=markup
                                    self.markup.inner_html+=f"<pre>{markups}</pre>"
                                    #break
                    self.results.inner_html+=delim+rmarkup  
                    delim="<br>" 
                    await self.wp.update()
            
        except Exception as ex:
            self.handleException(ex)
            
    async def onChangeMarkup(self,msg):
        """
        handle button to search for terms
        """
        try:
            self.markup_name=msg.value
        except Exception as ex:
            self.handleException(ex)
            
    def setupRowsAndCols(self):
        """
        setup the general layout
        """
        head_html="""<link rel="stylesheet" href="/static/css/md_style_indigo.css">"""
        self.wp=self.getWp(head_html)
        self.button_classes = """btn btn-primary"""
        # rows
        self.rowA=self.jp.Div(classes="row",a=self.contentbox)
        self.rowB=self.jp.Div(classes="row",a=self.contentbox)
        self.rowC=self.jp.Div(classes="row",a=self.contentbox)
        # columns
        self.colA1=self.jp.Div(classes="col-12",a=self.rowA)
        self.colB1=self.jp.Div(classes="col-6",a=self.rowB)
        self.rowB1r1=self.jp.Div(classes="row",a=self.colB1)
        self.colB11=self.jp.Div(classes="col-3",a=self.rowB1r1)
        self.rowB1r2=self.jp.Div(classes="row",a=self.colB1)
        self.colB12=self.jp.Div(classes="col-6",a=self.rowB1r2)
        self.colB2=self.jp.Div(classes="col-6",a=self.rowB)
        self.colC1=self.jp.Div(classes="col-12",a=self.rowC,style='color:black')
        # standard elements
        self.errors=self.jp.Div(a=self.colA1,style='color:red')
        self.messages=self.jp.Div(a=self.colC1,style='color:black')    
        
    async def onChangeLanguage(self,msg):
        """
        react on language being changed via Select control
        """
        self.language=msg.value  
        
    async def onChangeWikiUser(self,msg):
        """
        react on a the wikiuser being changed via a Select control
        """
        self.wikiId=msg.value
  
    def addLanguageSelect(self):
        """
        add a language selector
        """
        self.languageSelect=self.createSelect("Language","en",a=self.colB11,change=self.onChangeLanguage)
        for language in self.getLanguages():
            lang=language[0]
            desc=language[1]
            desc=html.unescape(desc)
            self.languageSelect.add(self.jp.Option(value=lang,text=desc))
            
    def addWikiUserSelect(self):
        """
        add a wiki user selector
        """
        if len(self.wikiUsers)>0:
            self.wikiuser_select=self.createSelect("wikiId", value=self.wikiId, change=self.onChangeWikiUser, a=self.colB11)
            for wikiUser in sorted(self.wikiUsers):
                self.wikiuser_select.add(self.jp.Option(value=wikiUser,text=wikiUser)) 
      
    async def settings(self)->"jp.WebPage":
        '''
        settings
        
        Returns:
            jp.WebPage: a justpy webpage renderer
        '''
        self.setupRowsAndCols()
        self.addLanguageSelect()
        self.addWikiUserSelect()
        return self.wp
    
    async def about(self)->"jp.WebPage":
        '''
        show about dialog
        
        Returns:
            jp.WebPage: a justpy webpage renderer
        '''
        self.setupRowsAndCols()
        self.aboutDiv=About(a=self.colB1,version=self.version)
        # @TODO Refactor to pyJustpyWidgets
        return self.wp
        
    async def content(self)->"jp.WebPage":
        '''
        provide the main content page
        
        Returns:
            jp.WebPage: a justpy webpage renderer
        '''
        self.setupRowsAndCols()
        self.results=self.jp.Div(a=self.colC1)
        self.markup=self.colB2
        # sotsog search
        self.markup_select = self.createSelect("markup",
            value=self.markup_name,
            change=self.onChangeMarkup,
            a=self.colB11)
        for markup_name in self.markup_names:
            self.markup_select.add(self.jp.Option(value=markup_name,text=markup_name))

        self.searchTerms=self.jp.Textarea(placeholder="enter search terms", a=self.colB12, rows=5,cols=120)
        self.searchButton=self.jp.Button(text="search",click=self.onSearchButton,a=self.colB12,classes=self.button_classes)
        return self.wp
    
    def start(self,host,port,debug):
        """
        start the server
        """
        self.debug=debug
        import justpy as jp
        jp.justpy(self.content,host=host,port=port)


'''
Created on 17.11.2022

@author: wf
'''
import skg
import datetime
import habanero
import habanero.cn as cn

class Crossref:
    """
    Crossref access
    """
    
    def __init__(self,mailto=None,ua_string=None):
        """
        constructor
        """
        if mailto is None:
            mailto="wf@bitplan.com"
        if ua_string is None:
            ua_string=f"pysotsog/{skg.__version__} (https://pypi.org/project/pysotsog/; mailto:{mailto})"
        self.cr = habanero.Crossref(mailto=mailto,ua_string=ua_string)  
    
    def doiMetaData(self, dois:list):
        """ 
        get the meta data for the given dois
        
        Args:
            doi(list): a list of dois
        """
        metadata = None
        response = self.cr.works(ids=dois)
        if 'status' in response and 'message' in response and response['status'] == 'ok':
            metadata = response['message']
        return metadata
    
    def doiBibEntry(self,dois:list):
        """
        get bib entries for the given dois
        """
        bibentry=cn.content_negotiation(ids = dois, format = "bibentry")
        return bibentry
    
    def asScite(self,meta_data:dict):
        """
        convert the given meta data to #Scite format
        
        see https://github.com/SemanticMediaWiki/SemanticCite/blob/master/src/FilteredMetadata/BibliographicFilteredRecord.php
        """
        def unlist(value):
            text=""
            delim=""
            for item in value:
                text+=f"{delim}{item}"
                delim=";"
            if len(value)>1:
                text+="|+sep=;"
            return text
        
        def get_author(value):
            author=""
            delim=""
            for arec in value:
                author+= f"""{delim}{arec["given"]} {arec["family"]}"""
                delim=";"
            return author
        
        timestamp=datetime.datetime.utcnow().strftime('%Y-%m-%d')
        ref_type="journal-article"
        title=meta_data["title"][0].lower()
        title_2=title[:2]
        author_lower=""
        if "author" in meta_data:
            author_lower=meta_data["author"][0]["family"].lower()
        year=""
        if "published-print" in meta_data:
            year=meta_data["published-print"]["date-parts"][0][0]
        reference=f"{author_lower}{year}{title_2}"
        markup=""
        for skey,mkey,func in [
            ("title","title",unlist),
            ("subtitle","subtitle",unlist),
            ("authors","author",get_author),
            ("journal","container-title",unlist),
            ("publisher","publisher",str),
            ("issn","ISSN",unlist),
            ("subject","subject",unlist),
            ("volume","volume",str),
            ("pages","page",str),
            ("doi","DOI",str)
        ]:
            if mkey in meta_data:
                value=meta_data[mkey]
                if value:
                    value=func(value)
                    markup+=f"\n|{skey}={value}"
        markup=f"""{{{{#scite:
|reference={reference}
|type={ref_type}{markup}
|year={year}
|retrieved-from=https://dx.doi.org/
|retrieved-on={timestamp}
}}}}"""
        full_markup=f"{title}\n[[CiteRef::{reference}]]\n{markup}"
        return full_markup


'''
Created on 2022-11-21

@author: wf
'''
from tests.basetest import Basetest
from skg.kg import SKG_Def

class BaseSkgTest(Basetest):
    """
    extend the Basetest
    """
    def setUp(self, debug=False, profile=True):
        Basetest.setUp(self, debug=debug, profile=profile)
        self.skg_def=SKG_Def()
    
    def check_id_examples(self,id_examples,createFunc,checkItem,debug:bool=False):
        """
        check the given examples
        
        Args:
            id_examples(list): a list of dict with examples
            createFunct(func): a function to be  used to create items
            checkItem(func): a function to be called for testing each item
            debug(bool): if True show debug information
        """
        for id_example in id_examples:
            id_name=id_example["id_name"]
            id_value=id_example["id_value"]
            id_concept=id_example["concept"]
            # use createFunc as a factory
            items=createFunc(id_concept,id_name,id_value)
            if debug:
                for item in items:
                    print(item)
            self.assertEqual(1,len(items))
            item=items[0]
            self.assertEqual(item.concept.name,id_concept.name)
            checkItem(item,id_name,id_value,debug)


'''
Created on 2022-11-17

@author: wf
'''
from lodstorage.sparql import SPARQL
from skg.owl import Owl

class Dblp:
    """
    Schloss Dagstuhl Dblp computer science bibliography
    """
    
    def __init__(self,endpoint:str="https://qlever.cs.uni-freiburg.de/api/dblp"):
        """
        constructor
        
        Args:
            endpoint(str): the endpoint to use
        """
        self.endpoint=endpoint
        self.schema=Owl("dblp","https://dblp.org/rdf/schema", "Wolfgang Fahl","2022-11-19")
        self.sparql=SPARQL(self.endpoint)
        
    
    def get_paper_records(self,regex:str,prop_name:str="title",limit:int=100,debug:bool=False)->list:
        """
        get papers fitting the given regex
        
        Args:
            prop_name(str): the property to filter
            regex(str): the regex to filter for
            limit(int): the maximum number of records to return
            debug(bool): if True show debug information
            
        Returns:
            list: a list of dict of paper records
        """
        sparql_query="""PREFIX dblp: <https://dblp.org/rdf/schema#>
PREFIX xsd: <http://www.w3.org/2001/XMLSchema#>
SELECT
  ?paper 
  ?year
  ?yearofevent
  #?month
  ?doi
  ?isbn
  ?title
  (GROUP_CONCAT(?author_o) as ?authors)
  ?publishedin
WHERE {
  ?paper dblp:title ?title .
  ?paper dblp:doi ?doi .
  OPTIONAL { ?paper dblp:yearOfEvent ?yearofevent } .
  OPTIONAL { ?paper dblp:isbn ?isbn }.
  ?paper dblp:authoredBy ?author_o.
  ?paper dblp:publishedIn ?publishedin .
  ?paper dblp:yearOfPublication ?year.
  OPTIONAL { ?paper dblp:monthOfPublication ?month}.
"""
        sparql_query+=f"""FILTER regex(?{prop_name}, "{regex}").\n"""
        sparql_query+=f"""
}}
GROUP BY 
  ?paper 
  ?title 
  ?doi 
  ?isbn
  ?year 
  ?yearofevent
  ?month 
  ?publishedin 
ORDER BY DESC(?year)
LIMIT {limit}"""
        if debug:
            print(sparql_query)
        records=self.sparql.queryAsListOfDicts(sparql_query)
        return records
    
    def get_random_papers(self,year:int=2020,limit:int=10):
        sparql_query=f"""PREFIX dblp: <https://dblp.org/rdf/schema#>
SELECT 
  ?paper 
  (SAMPLE(?doi_o) as ?doi)
  (SAMPLE(?title_o) as ?title)
  (MIN(?year_o) as ?year)
  (GROUP_CONCAT(?author_o) as ?authors)
  (SAMPLE(?publishedin_o) as ?publishedin)
  (SAMPLE(?sortKey) as ?sortKey)
WHERE {{
  VALUES ?year_o {{ "{year}" }}
  ?paper dblp:title ?title_o .
  ?paper dblp:doi ?doi_o .
  ?paper dblp:authoredBy ?author_o.
  ?paper dblp:publishedIn ?publishedin_o .
  ?paper dblp:yearOfPublication ?year_o.
  BIND(RAND() AS ?sortKey)
}}
GROUP BY ?paper
ORDER BY ?sortKey 
LIMIT {limit}
        """
        


'''
Created on 22.11.2022

@author: wf
'''
from wikibot.wikiuser import WikiUser
from wikibot.wikiclient import WikiClient
from wikibot.smw import SMWClient
from skg.wikidata import Wikidata
class SemWiki:
    """
    access to Semantic mediawiki
    """
    
    def __init__(self,wikiUser:WikiUser,withLogin:bool=False):
        """
        
        constructor
        
        Args:
            wikiUser:WikiUser
        """
        self.wikiUser=wikiUser
        self.wikiClient=WikiClient.ofWikiId(wikiUser.wikiId)
        if withLogin:
            self.wikiClient.login()
        self.smw=SMWClient(self.wikiClient.getSite())
    
        
    def id_refs(self,mainlabel="pageTitle",condition="DOI::+",title:str="DOI references",askExtra:str="",id_prop="DOI",id_name="doi")->list:
        """
        get a list of DOI references from the given wiki
        """
        ask = f"""{{{{#ask:[[{condition}]]{askExtra}
|?{id_prop}={id_name}
|mainlabel={mainlabel}
|?Creation_date=creationDate
|?Modification_date=modificationDate
|?Last_editor_is=lastEditor
}}}}
"""
        refs=self.smw.query(ask,title)
        return refs
            
    def papers(self):
        """
        get the paper records
        """
        askExtra="""\n|?Citation_text=reference"""
        paper_records=self.id_refs(condition="Citation_text::+",title="doi paper referencs", askExtra=askExtra)
        return paper_records
    
    def scholars(self):
        """
        get scholars
        """
        condition="Concept:Scholar"
        mainlabel="Scholar"
        askExtra="""
| ?Scholar name = name
| ?Scholar firstName = firstName
| ?Scholar homepage = homepage
| ?Scholar orcid = orcid
| ?Scholar dblpId = dblpId
| ?Scholar googleScholarUser = googleScholarUser
| ?Scholar linkedInId = linkedInId
| ?Scholar gndId = gndId
| ?Scholar smartCRMId = smartCRMId
|sort=Scholar name,Scholar firstName
|order=ascending,ascending
"""       
        scholars=self.id_refs(mainlabel, condition, "scholars", askExtra, "Scholar wikiDataId", "wikiDataId")
        return scholars
    
    @classmethod
    def asMarkup(self,scholar)->str:
        """
        return the markup for the given scholar
        
        Args:
            scholar(Node): the scholar
        Returns:
            str: the semantic mediawiki markup
        """
        markup="{{Scholar"
        
        for prop_name,prop in scholar.concept.props.items():
            if prop.hasmap("smw"):
                smw_prop=prop.getmap("smw")
                if hasattr(scholar,prop_name):
                    value=getattr(scholar,prop_name)
                    # @TODO refactor
                    qid=Wikidata.getQid(value)
                    if value!=qid:
                        # potential lookup need
                        if prop_name!="wikiDataId":
                            value=Wikidata.getLabelForQid(qid)
                        else:
                            value=qid
                    markup+=f"\n|{smw_prop}={value}"
        markup+="\n}}"
        return markup
        


'''
Created on 2022-11-22

@author: wf
'''
from tests.basetest import Basetest
from skg.doi import DOI
from skg.smw import SemWiki
from wikibot.wikiuser import WikiUser
import json
from collections import Counter

class TestSMW(Basetest):
    """
    test Semantic Mediawiki access
    """
    
    def setUp(self, debug=False, profile=True):
        Basetest.setUp(self, debug=debug, profile=profile)
        self.wikiUsers=WikiUser.getWikiUsers()
        
    
    def checkWikis(self,wikis,accessor,verbose:bool=True,debug:bool=False):
        """
        check the given wikis
        
        Args:
            wikis(list): a list of wikiId,withLogin tuples
            accessor(func): the function to check
            verbose(bool): if True show verbose details
            debug(bool): if True show debugging information
        """
        doiCounter=Counter()
        records=None
        for wikiId,withLogin in wikis:
            if wikiId in self.wikiUsers:
                wikiUser=self.wikiUsers[wikiId]
                semwiki=SemWiki(wikiUser,withLogin=withLogin)
                records=accessor(semwiki)
                if verbose:
                    print(f"found {len(records)} {accessor.__name__} references in {wikiId}" )
                if debug:
                    print(json.dumps(records,indent=2,default=str))
                for key,record in records.items():
                    if "doi" in record:
                        doi=record["doi"]
                        if doi:
                            self.assertTrue(DOI.isDOI(doi),f"{key}:doi={doi}")
                            doiCounter[wikiId]+=1
        if verbose:
            print(doiCounter.most_common())
        return records
    
    def test_dois(self):
        """
        test doi access
        """
        debug=self.debug
        wikis=[("confident","False)")]
        self.checkWikis(wikis,SemWiki.id_refs,debug=debug)
               
    def test_papers(self):
        """
        test getting paper references from a Semantic MediaWiki
        """
        debug=self.debug
        wikis=[("ceur-ws",False),("rq",True),("media",True)]
        self.checkWikis(wikis,accessor=SemWiki.papers,debug=debug)
                
    def test_scholars(self):
        """
        check access to scholars
        """
        debug=self.debug
        #debug=True
        wikis=[("media",True)]
        self.checkWikis(wikis,accessor=SemWiki.scholars,debug=debug)  
            
        
    



'''
Created on 2022-11-22

@author: wf
'''
# see https://pypi.org/project/semanticscholar/
from semanticscholar import SemanticScholar as SemScholar

class SemanticScholar:
    """
    wrapper for Semantic Scholar API
    """
    
    def __init__(self):
        """
        constructor
        """
        self.sch = SemScholar()
        
    def get_paper(self,doi:str):
        """
        get the paper with the given DOI identifier
        """
        paper=self.sch.get_paper(doi)
        return paper
    
    def get_author(self):
        """
        https://api.semanticscholar.org/api-docs/graph#tag/Author-Data/operation/get_graph_get_author_search
        """
        pass
        


'''
Created on 2022-11-16

@author: wf
'''
from tests.basetest import Basetest
from skg.sotsog import SotSog
from skg.search import SearchOptions
class TestSotsog(Basetest):
    """
    test concerning the SotSog main class
    """

    def test_sotsog_search(self):
        """
        test searches
        """
        sotsog=SotSog()
        search_examples=[
            {
                "search": ["10.1145/1963405.1963408"],
                "qid": "Q55693406", 
                "concept": "Paper"
            },
            {
                "search": ["RWTH"],
                "qid": "Q273263",
                "concept": "Institution"
            }, 
            {
                "search": ["Wikidata Workshop 2022"],
                "qid": "Q112055391", 
                "concept": "Event"
            }, 
            {
                "search": ["VNC"],
                "qid": "Q105695678", 
                "concept": "EventSeries"
            }, 
            {
                "search": ["Albert","Einstein"],
                "qid": "Q937",
                "concept":  "Scholar"
            },
            {
                "search": ["Designing the web for an open society"],
                "qid": "Q55693406", 
                "concept": "Paper"
            },
        ]
        debug=self.debug
        debug=True
        for _i,search_example in enumerate(search_examples):
            search=search_example["search"]
            qid=search_example["qid"]
            concept_name=search_example["concept"]
            options=SearchOptions(show=False, open_browser=False)
            s_result=sotsog.search(search,options)        
            for item in s_result.items:
                if debug:
                    print(item)
            for item in s_result.items:
                self.assertEqual(concept_name,item.concept.name)
                if hasattr("item", "wikiDataId"):
                    self.assertTrue(qid in item.wikiDataId)
                # @TODO compare properties against samples
                
    def test_get_markups(self):
        """
        get markups for a given DOI
        """
        sotsog=SotSog()
        options=SearchOptions(show=False, open_browser=False)
        dois=["10.7287/peerj.preprints.27466v1"]
        #@TODO test search via DOI, ORCID and so on



'''
Created on 2022-11-19

@author: wf
'''

class SearchOptions:
    """
    wrapper for search results
    """
    def __init__(self,limit:int=9,lang='en',show:bool=True,
               markup_names=["bibtex"],open_browser:bool=False):
        """
        constructor
        
        Args:
            limit(int): limit for the maximum number of results
            lang(str): the language code to use for the search
            show(bool): if True print the search results
            markup_names(list): a list of markup names to support
            open_browser(bool): if True open a browser for the target page of the item e.g. scholia

        """
        self.limit=limit
        self.lang=lang
        self.show=show
        self.markup_names=markup_names
        self.open_browser=open_browser
        
class SearchResult:
    """
    wrapper for search results
    """
    def __init__(self,search_list:list,options=SearchOptions):
        """
        constructor
        
         Args:
            search_list(list): a list of search terms
            options(SearchOptions): the search options to apply
        """
        self.search_list=search_list
        self.options=options
        self.items=[]


'''
Created on 2022-11-22

@author: wf
'''
import json
from rdflib.namespace import OWL
from skg.schema import Schema
import rdflib
from skg.profiler import Profiler

class Owl(Schema):
    """
    Web Ontology Language access
    see https://en.wikipedia.org/wiki/Web_Ontology_Language
    """
    
    def __init__(self,name:str,url:str,authors:str,inception:str):
        """
        constructor
            
        Args:
            name(str): the name of this schema
            url(str): the url of this schema
            authors(str): the authors of this schema
            inception(str): the inception of this schema
        """
        Schema.__init__(self,name,url,authors,inception)
        self.schema_url=url
        self.schema=None
        
    def show_triples(self,result):
        """
        show the triples for the given query result
        """
        for i,row in enumerate(result):
            print(f"{i+1}:{row}")
        
    def query_schema(self,query:str,formats:str="",profile:bool=False):
        """
        query the schema
        
        Args:
            query(str): the SPARQL query to execute
            formats(str): if "triples" is in th format string show the results string
            profile(bool): if True show timing information for the query
        """
        profiler=Profiler(f"query {query}",profile=profile)
        result=self.schema.query(query)
        if "triples" in formats:
            self.show_triples(result)
        if profile:
            profiler.time(f" for {len(result)} triples")
        return result    
        
    def loadSchema(self,formats:str="",profile:bool=False):
        """
        load the schema
        
        Args:
            formats(str): the formats to dump
            profile(bool): if True show timing
        """
        # https://stackoverflow.com/questions/56631109/how-to-parse-and-load-an-ontology-in-python
        profiler=Profiler(f"reading {self.name} schema",profile=profile)
        self.schema = rdflib.Graph()
        self.schema.parse (self.schema_url, format='application/rdf+xml')
        if profile:
            profiler.time(f" for {len(self.schema)} triples")
        for t_format in formats.split(","):
            if t_format and t_format!="triples":
                print (self.schema.serialize(format=t_format))
        self.schema.bind('owl',OWL)
        query = """select distinct ?s ?p ?o 
where { ?s ?p ?o}
"""
        self.query_schema(query,formats=formats,profile=profile)
        return self.schema
        
    def unprefix_value(self,value:object,prefixes:list=["http://xmlns.com/foaf/0.1/"])->str:
        """
        get rid of RDF prefixes to simplify our life
        
        Args:
            value(object): the RDFLib value to unprefix
            prefixes(list): list of prefixes to remove
        Returns:
            str: a simple string representation
        """
        if isinstance(value,list):
            if len(value)>=1:
                    value=value[0]
        if isinstance(value,dict):
            for akey in ["@id","@value"]:
                if akey in value:
                    value=value[akey]
        if isinstance(value,str):
            parts=value.split("#")
            if len(parts)==2:
                value=parts[1]
        for prefix in prefixes:
            if value.startswith(prefix):
                value=value.replace(prefix,"")
        return value
    
    def unprefix_row(self,row:dict):
        """
        get rid of the RDF prefixes in keys and values of the given row
        to simplify our life
        
        Args:
            row(dict): a dict of RDF values to unprefix
        """
        for key in list(row.keys()):
            org_value=row[key]
            value=self.unprefix_value(org_value)
            row[key]=value
            if "#" in key:
                noprefix_key=self.unprefix_value(key)
                row[noprefix_key] = row.pop(key)
            row[f"{key}_rdf"]=org_value
    
    def toClasses(self):
        """
        convert to a classes dict of dicts
        
        Returns:
            dict: a dict of dictionaries
        """
        json_ld=self.schema.serialize(format="json-ld")
        schema_dict=json.loads(json_ld)
        classes={}
        # get rid of prefixes
        for row in schema_dict:
            self.unprefix_row(row)
        # pass 1 - classes
        for row in schema_dict:
            name=row["@id"]
            ptype=row["@type"]
            comment=row.get("comment","")
            label=row.get("label","")
            subClassOf=row.get("subClassOf","")
            if ptype=="Class":
                if name in classes:
                    clazz=classes[name]
                else:
                    clazz={
                        "@comment":comment,
                        "@label": label,
                        "@subClassOf": subClassOf
                    }
                    classes[name]=clazz
        # pass 2 - properties
        for row in schema_dict:
            name=row["@id"]
            ptype=row["@type"]
            comment=row.get("comment","")
            domain=row.get("domain","")
            prange=row.get("range","")
            plabel=row.get("label")
            if ptype=="Property":
                prop={
                    "name": name,
                    "comment": comment,
                    "label": plabel,
                    "domain": domain,
                    "range": prange
                }
                if domain in classes:
                    clazz=classes[domain]
                    clazz[name]=prop
            pass
        wrapped_classes={
            "classes":classes
        }
        return wrapped_classes



'''
Created on 2022-11-22

@author: wf
'''
from tests.basetest import Basetest
from skg.doi import DOI
from skg.dblp import Dblp
from dataclasses import dataclass

@dataclass
class DOIExample:
    doi: str
    isDoi: bool
    
class TestDOI(Basetest):
    """
    test DOI access
    """
    
    
    def testDOICheck(self):
        """
        check DOI regex
        """
        doi_examples=[DOIExample("10.1007/11581116_19",True),DOIExample("xyz",False)]
        for doi_example in doi_examples:
            self.assertTrue(DOI.isDOI(doi_example.doi)==doi_example.isDoi,doi_example.doi)
                            
    def test_dblp_dois(self):
        """
        test dblp dois
        """
        dblp=Dblp()
        limit=5
        debug=True
        paper_records=dblp.get_paper_records("CEUR Workshop Proceedings","publishedin",limit=limit,debug=debug)
        for paper_record in paper_records:
            print(paper_record)


'''
Created on 2022-11-22

@author: wf
'''
import datetime

class Schema():
    """
    a schema
    """
    
    def __init__(self,name:str,url:str,authors:str,inception:str):
        """
        constructor
            
        Args:
            name(str): the name of this schema
            url(str): the url of this schema
            authors(str): the authors of this schema
            inception(str): the inception of this schema
        """
        self.name=name
        self.url=url
        self.authors=authors
        self.inception=inception
        
    def classesToPlantUml(self,classes:dict,indent:str="  "):
        """
        convert the given classes dict to plantuml
        
        Args:
            classes(dict): a dictionary of classes
            indent(str): the indentation to apply
        """
        classes=classes["classes"]
        markup=""
        for cname,clazz in classes.items():
            class_markup=""
            rel_markup="" # relations
            for pname,prop in clazz.items():
                if pname.startswith("@"):
                    pass
                else:
                    prange=prop['range']
                    if prange in classes:
                        #  Class01 "1" *-- "many" Class02 : contains
                        rel_markup+=f"{indent}{cname}--{prange}:{pname}\n"
                    else:
                        class_markup+=f"{indent}  {pname}:{prange}\n"
            class_markup=f"{indent}class {cname}{{\n{class_markup}\n{indent}}}\n"
            class_markup+=rel_markup
            if "@subClassOf" in clazz:
                general=clazz["@subClassOf"]
                if general:
                    class_markup+=f"{indent}{general} <|-- {cname}\n"
            note=f"{indent}note top of {cname}\n"
            if "@label" in clazz:
                note+=f"""{indent}{clazz["@label"]}\n"""
            if "@comment" in clazz:
                note+=f"""{indent}{clazz["@comment"]}\n"""
            note+=f"{indent}end note\n"
            class_markup=note+class_markup
            markup+=class_markup
        return markup
        
    def toPlantUml(self,header=None,footer=None):
            """
            get a plantuml version of the schema
            
            Args:
                header(str): the header to apply
                footer(str): the footer to apply
            
            Returns:
                str: the plantuml markup
            """
            timestamp=datetime.datetime.utcnow().strftime('%Y-%m-%d')
            if header is None:
                header=f"""/'
     {self.authors} {self.inception}
     updated {timestamp}
      
     {self.name} {self.schema_url}
     converted from owl to plantuml
    '/
    title  {self.name} schema {self.schema_url} converted from owl to plantuml updated {timestamp}
    hide circle
    package foaf {{
      class Document {{
      }}
    }}
    package dblp {{
     """
            if footer is None:
                footer="}\n"
            classes=self.toClasses()
            markup=header+self.classesToPlantUml(classes,indent="  ")+footer
            return markup


'''
Created on 2022-11-16

@author: wf
'''

import os
import webbrowser
import sys
import traceback
from skg.version import Version
from argparse import ArgumentParser
from argparse import RawDescriptionHelpFormatter
from skg.wdsearch import WikidataSearch
from skg.wikidata import Wikidata
from skg.smw import SemWiki
from skg.kg import SKG_Def
from skg.graph import Node
from skg.doi import DOI
from skg.crossref import Crossref
from skg.skgbrowser import SkgBrowser
from skg.search import SearchOptions, SearchResult
from jpcore.justpy_app import JustpyServer

class SotSog():
    """
    Standing on the shoulders of giants
    """
    
    def __init__(self,args=None):
        """
        constructor
        
        Args:
            args(): command line Arguments(optional)
        """
        self.args=args
        debug=False
        if args:
            debug=args.debug
        self.debug=debug
        Node.debug=debug
        self.wikipedia_url="https://en.wikipedia.org/wiki/Standing_on_the_shoulders_of_giants"
        self.skg_def=SKG_Def()
        self.scholar_concept=self.skg_def.concepts["Scholar"]
    
    def getMarkups(self,item,options:SearchOptions)->dict:
        """
        get the markups for the given item and search options
        
        Args:
            item(Node): the item to get the markup for
            options(SearchOptions): the search options to apply
        """
        markups={}
        do_markup=len(options.markup_names)>0
        if do_markup:
            if item.concept.name=="Paper":
                doi=getattr(item, "doi",None)
                if doi is not None:
                    crossref=Crossref()
                    if "bibtex" in options.markup_names:
                        bibentry=crossref.doiBibEntry([doi])
                        markups["bibtex"]=bibentry
                    if "scite" in options.markup_names:
                        meta_data=crossref.doiMetaData([doi])
                        scite_entry=crossref.asScite(meta_data)
                        markups["scite"]=scite_entry
            if item.concept.name=="Scholar":
                if "smw" in options.markup_names:
                    markups["smw"]=SemWiki.asMarkup(item)
        return markups
    
    def wd_search(self,wd:Wikidata,search_term:str,options)->list:
        """
        do a wikidata search
        """
        items=[]
        wds=WikidataSearch(language=options.lang,debug=self.debug)
        search_options=wds.searchOptions(search_term,limit=options.limit)
        qids=[]
        for qid,itemLabel,desc in search_options:
            qids.append(qid)
        class_map=wd.getClassQids(qids)
        for qid,itemLabel,desc in search_options:
            if qid in class_map:
                class_rows=class_map[qid]
                for class_row in class_rows:
                    class_qid=class_row["class_qid"]
                    concept=self.skg_def.conceptForQid(class_qid)
                    if concept is not None:
                        wd_items=concept.cls.from_wikidata_via_id(concept,"wikiDataId", qid, lang=options.lang)
                        if len(wd_items)>0:
                            item=wd_items[0]
                            items.append(item)
                            self.handleItem(item,qid,itemLabel,desc,options)
        return items                    
                            
    def handleItem(self,item,item_id,itemLabel,desc,options):
        """
        handle the given item as a search result
        """
        if options.show:
            print(f"{itemLabel}({item_id}):{desc}✅")
            print(item)
        item.markups=self.getMarkups(item,options)
        if options.show:
            for markup_name,markup in item.markups.items():
                print(f"{markup_name} markup:")
                print(markup)
            pass
        if options.open_browser:
            browser_url=item.browser_url()
            if browser_url is not None:
                print(f"opening {browser_url} in browser")
                webbrowser.open(browser_url)

    def search(self,search_list,options:SearchOptions)->SearchResult:
        """
        search with the given search list
        
        Args:
            search_list(list): a list of search terms
            options(SearchOptions): the search options to apply
        """
        search_result=SearchResult(search_list,options)
        search_term=' '.join(search_list)
        wd=Wikidata(debug=self.debug)
        if DOI.isDOI(search_term):
            # DOI may not be referencing paper but something else
            paper_concept=self.skg_def.concepts["Paper"]
            items=Node.from_wikidata_via_id(paper_concept, "doi", search_term, options.lang)
            for item in items:
                item_id=item.wikiDataId
                itemLabel=item.label
                desc="?"
                self.handleItem(item, item_id, itemLabel, desc, options)
            dblp_items=Node.from_dblp_via_id(paper_concept, "doi", search_term)
            for item in dblp_items:
                item_id=item.doi
                itemLabel=item.title
                desc=item.title
                self.handleItem(item, item_id, itemLabel, desc, options)
            items.extend(dblp_items)
        else:
            items=self.wd_search(wd,search_term,options)               
        search_result.items=items
        return search_result

__version__ = Version.version
__date__ = Version.date
__updated__ = Version.updated


def main(argv=None): # IGNORE:C0111
    '''main program.'''

    if argv is None:
        argv=sys.argv[1:]
    
    program_name = os.path.basename(__file__)
    program_shortdesc = Version.description
    
    program_version =f"v{__version__}" 
    program_build_date = str(__updated__)
    program_version_message = f'{program_name} ({program_version},{program_build_date})'

    user_name="Wolfgang Fahl"
    program_license = '''%s

  Created by %s on %s.
  Copyright 2022 Wolfgang Fahl. All rights reserved.

  Licensed under the Apache License 2.0
  http://www.apache.org/licenses/LICENSE-2.0

  Distributed on an "AS IS" basis without warranties
  or conditions of any kind, either express or implied.

USAGE
''' % (program_shortdesc, user_name,str(__date__))
    try:
        # Setup argument parser
        parser = ArgumentParser(description=program_license, formatter_class=RawDescriptionHelpFormatter)
        parser.add_argument('search', action='store', nargs='*', help="search terms")
        parser.add_argument("--about",help="show about info",action="store_true")
        parser.add_argument("--bibtex",help="output bibtex format",action="store_true")
        parser.add_argument("-d", "--debug", dest="debug", action="store_true", help="show debug info")
        parser.add_argument('--host',default=JustpyServer.getDefaultHost())
        parser.add_argument("-la", "--lang",help="language code to use",default="en")
        parser.add_argument("-li", "--limit",help="limit the number of search results",type=int,default=9)
        parser.add_argument("-nb","--nobrowser",help="do not open browser",action="store_true")
        parser.add_argument('--port',type=int,default=8765)
        parser.add_argument("--scite",help="output #scite format",action="store_true")
        parser.add_argument("--smw",help="output Semantic MediaWiki (SMW) format",action="store_true")   
        parser.add_argument("--serve",help="start webserver",action="store_true")
        parser.add_argument('-V', '--version', action='version', version=program_version_message)
        args = parser.parse_args(argv)
        if len(argv) < 1:
            parser.print_usage()
            sys.exit(1)
        sotsog=SotSog(args)
        markup_names=[]
        if args.bibtex: markup_names.append("bibtex")
        if args.scite: markup_names.append("scite")
        if args.smw: markup_names.append("smw")
        options=SearchOptions(limit=args.limit,lang=args.lang,
                      markup_names=markup_names,
                      open_browser=not args.nobrowser)
        if args.about:
            print(program_version_message)
            print(f"see {Version.doc_url}")
            webbrowser.open(Version.doc_url)
        elif args.serve:
            skgBrowser=SkgBrowser(version=Version,sotsog=sotsog,options=options)
            url=f"http://{args.host}:{args.port}"
            webbrowser.open(url)
            skgBrowser.start(args.host, args.port,debug=args.debug)
            pass
        else:
            sotsog.search(args.search,options)
        pass
    except KeyboardInterrupt:
        ### handle keyboard interrupt ###
        return 1
    except Exception as e:
        if DEBUG:
            raise(e)
        indent = len(program_name) * " "
        sys.stderr.write(program_name + ": " + repr(e) + "\n")
        sys.stderr.write(indent + "  for help use --help")
        if args.debug:
            print(traceback.format_exc())
        return 2       
        
DEBUG = 1
if __name__ == "__main__":
    if DEBUG:
        sys.argv.append("-d")
    sys.exit(main())



'''
Created on 2022-11-16

@author: wf
'''
from skg.scholar import Scholar,Institution
from skg.paper import Paper
from skg.event import Event,EventSeries,Proceedings
from skg.location import Country
from skg.graph import Concept

class SKG_Def:
    """
    scholary knowledge graph
    """
    
    def __init__(self):
        """
        constructor
        """
        self.concepts={
            # main concepts
            "Scholar": Concept(name="Scholar",cls=Scholar),
            "Institution": Concept(name="Institution",cls=Institution),
            "Paper": Concept(name="Paper",cls=Paper),
            "Event": Concept(name="Event",cls=Event),
            "EventSeries": Concept(name="EventSeries",cls=EventSeries),
            "Proceedings": Concept(name="Proceedings",cls=Proceedings),
            # neighbour concepts
            "Country": Concept(name="Country",cls=Country)
        }
        self.concepts["Scholar"].map_wikidata("Q5","author",[
            ("name","label"),
            ("dblpId","P2456"),
            ("gndId","P227"),
            ("linkedInId","P6634"),
            ("homepage","P856"),
            ("googleScholarUser","P1960"),
            ("orcid","P496"),
            ("givenName","P735"),
            ("familyName","P734"),
            ("gender","P21"),
            ("image","P18"),
            ("occupation","P106"),
            ("Semantic_Scholar_author_ID","P4012")
        ]).map("dblp",[
            ("name","primaryCreatorName"),
            ("homepage","primaryHomepage"),
            ("orcid","orcid")
        ]).map("smw",[
            ("wikiDataId","wikiDataId"),
            ("familyName","name"),
            ("givenName","firstName"),
            ("googleScholarUser","googleScholarUser"),
            ("homepage","homepage"),
            ("dblpId","dblpId"),
            ("orcid","orcid"),
            ("linkedInId","linkedInId")
        ])
        self.concepts["Institution"].map_wikidata("Q4671277","organization",[
            ("short_name","P1813"), # 2.0 % 
            ("inception","P571"), # 65.8 %
            ("image","P18"), # 15.2 % 
            ("country","P17"), # 88.8 %
            ("located_in","P131"), # 51.9 %
            ("official_website","P856"), # 59.1%
            ("coordinate_location","P625") # 44.0 %
        ])  
        self.concepts["Paper"].map_wikidata("Q13442814","work",[
            ("title","label"),
            ("doi","P356"),
            ("DBLP_publication_ID","P8978"),
            ("publication_date","P577")
        ]).map("dblp",[
            ("title","title"),
            ("doi","doi")
        ])
        # scientific event
        self.concepts["Event"].map_wikidata("Q52260246","event",[
            ("title","P1476"),
            ("country","P17"), # 93.9% -> Human Settlement
            ("location","P276"), # 94.6%
            ("point_in_time","P585"),
            ("official_website","P856")
        ])
        # academic event series
        self.concepts["EventSeries"].map_wikidata("Q47258130","event-series",[
            ("title","P1476"), # 96.7 %
            ("short_name","P1813"), # 93.1 %
            ("VIAF_ID","P214"), # 60.5 %
            ("DBLP_venue_ID","P8926"), # 96.4 %
            ("gndId","P227"), #42.3 %
            ("inception","P571"), # 22.3 %
            ("official_website","P856") # 13.5 %
        ])
        # proceedings
        self.concepts["Proceedings"].map_wikidata("Q1143604","venue",[
            ("title","P1476"),
            ("short_name","P1813"),
            ("full_work_available_at_URL","P953"),
            ("publication_date","P577")
        ])
        # country
        self.concepts["Country"].map_wikidata("Q6256","topic",[
            ("name","label"), # 100% ?
            ("homepage","P856"), # 49.4%
            ("population","P1082"), # 57.4%
            ("capital","P36"), #59.8%
            ("coordinate_location","P625"), #58.6%
            ("iso_code","P297") # 53.3%
            
        ])
                      
        self.concepts_by_qid={}
        for concept in self.concepts.values():
            if concept.wd_class in self.concepts_by_qid:
                raise Exception(f"duplicate wd_class definition: {concept.wd_class}")
            self.concepts_by_qid[concept.wd_class]=concept
        
    def conceptForQid(self,qid:str)->Concept:
        """
        get the concept for the given wikidata Q Identifieer
        
        Args:
            qid(str): get the concept for the given Qid
            
        Return:
            Concept: or None if none is found
        """
        concept=self.concepts_by_qid.get(qid,None)
        return concept
        


'''
Created on 2022-11-17

@author: wf
'''
from tests.basetest import Basetest
from skg.crossref import Crossref
import json
from dataclasses import dataclass

@dataclass
class Example:
    doi:str
    author:str
        
class TestCrossref(Basetest):
    """
    test crossref access
    """
    
    def setUp(self, debug=False, profile=True):
        Basetest.setUp(self, debug=debug, profile=profile)
        self.crossref=Crossref()
        
    
    def test_crossref(self):
        """
        test crossref
        """  
        debug=self.debug
        debug=True
        doi_examples=[
            Example("10.1007/11581116_19","Atanas Kiryakov"),
            Example("10.1016/J.ARTMED.2017.07.002","Jean-Baptiste Lamy"),
            Example("10.1145/2882903.2899389","Rihan Hai")
        ]
        for example in doi_examples:
            doi=example.doi
            bib_entry=self.crossref.doiBibEntry(doi)
            if debug:
                print(bib_entry)
            expected=f"author = {{{example.author}"
            self.assertTrue(expected in bib_entry,expected)
            meta_data=self.crossref.doiMetaData(doi)
            if debug:
                print(json.dumps(meta_data,indent=2))
                self.assertTrue("DOI" in meta_data)
                self.assertEqual(doi.lower(),meta_data["DOI"])
                scite_entry=self.crossref.asScite(meta_data)
            if debug:
                print(scite_entry)

    def test_cookies(self):
        """
        test session cookies default
        """
        import requests
        session = requests.Session()
        print(session.cookies.get_dict())
        



'''
Created on 2022-11-22

@author: wf
'''
from tests.basetest import Basetest
from skg.semantic_scholar import SemanticScholar
import json

class TestSemanticScholar(Basetest):
    """
    test Semantic Scholar Access
    """
    
    def setUp(self, debug=False, profile=True):
        Basetest.setUp(self, debug=debug, profile=profile)
        self.semscholar=SemanticScholar()
        
    def showResults(self,results):
        print(results.total)
        for result in results:
            print(result)
        pass
        
    def testGetPaper(self):
        """
        test getting a paper
        """
        doi="10.1007/978-3-030-49461-2"
        paper=self.semscholar.get_paper(doi)
        print(json.dumps(paper.raw_data,indent=2,default=str))
        pass
    
    def testSearchAuthor(self):
        """
        test author search
        """
        results=self.semscholar.sch.search_author("Tim Berners-Lee")
        self.showResults(results)
    
    def testSearchPaper(self):
        """
        test paper search
        """
        titles=["M. Agosti, C. Thanos (Eds). Post-proceedings of the First Italian Research Conference on Digital Library Management Systems (IRCDL 2005), Padova, 28th January, 2005. September 2005."]
        for title in titles:
            results=self.semscholar.sch.search_paper(title)
            self.showResults(results)
        


'''
Created on 2022-11-17

@author: wf
'''
from tests.base_skg_test import BaseSkgTest
from skg.dblp import Dblp
from skg.graph import Node
import json

class TestDblp(BaseSkgTest):
    """
    test dblp access
    """
    
    def setUp(self, debug=False, profile=True):
        BaseSkgTest.setUp(self, debug=debug, profile=profile)
        self.dblp=Dblp()
    
    def test_dblp_papers(self):
        """
        test dblp paper access
        """
        sparql_query="""
PREFIX dblp: <https://dblp.org/rdf/schema#>
SELECT 
  ?paper 
  (SAMPLE(?doi_o) as ?doi)
  (SAMPLE(?title_o) as ?title)
  (MIN(?year_o) as ?year)
  (GROUP_CONCAT(?author_o) as ?authors)
  (SAMPLE(?publishedin_o) as ?publishedin)
WHERE {
  ?paper dblp:title ?title_o .
  ?paper dblp:doi ?doi_o .
  ?paper dblp:authoredBy ?author_o.
  ?paper dblp:publishedIn ?publishedin_o .
  ?paper dblp:yearOfPublication ?year_o.
}
GROUP BY ?paper
ORDER BY DESC(?year)
LIMIT 10
        """
        # rows since the query above returns truly tabular results
        paper_rows=self.dblp.sparql.queryAsListOfDicts(sparql_query)
        debug=self.debug
        debug=True
        if debug:
            for row in paper_rows:
                print(row)
        
    def test_dblp_schema(self):
        """
        test loading the dblp schema
        """
        schema=self.dblp.schema
        schema.loadSchema(formats="n3,json-ld") # xml
        classes=schema.toClasses()
        debug=self.debug
        debug=True
        if debug:
            print(json.dumps(classes,indent=2))
        classes=classes["classes"]
        self.assertTrue("Entity" in classes)
        entity=classes["Entity"]
        self.assertTrue("@subClassOf" in entity)
        self.assertEqual("Thing",entity["@subClassOf"])
        
    def test_uml(self):
        """
        test getting uml markup
        """
        schema=self.dblp.schema
        schema.loadSchema()
        uml_markup=schema.toPlantUml()
        debug=True
        if debug:
            print(uml_markup)
            
    def test_dblp_item_via_id_search(self):
        """
        test getting papers by id from dblp
        """
        debug=self.debug
        debug=True
        paper_concept=self.skg_def.concepts["Paper"]
        author_concept=self.skg_def.concepts["Scholar"]
        id_examples=[
            {
                "id_name": "orcid",
                "id_value": "0000-0003-1279-3709",
                "concept": author_concept
            },
            {
                "id_name": "doi",
                "id_value": "10.1007/978-3-031-19433-7_21",
                "concept": paper_concept
            }
        ]
        
        def checkItem(item:Node,id_name:str,id_value:str,debug:bool=False):
            """
            check the given item
            
            Args:
                item(Node): the item to check
                id_name(str): the name of the id used to retrieve the item
                id_value(str) the value that has been used to retriebe the item
                debug(bool): if True show debug information
            """
            if id_name=="doi":
                self.assertEqual(f"http://dx.doi.org/{id_value}",item.doi)
        
        self.check_id_examples(id_examples, createFunc=Node.from_dblp_via_id,checkItem=checkItem,debug=debug)


'''
Created on 2022-11-16

@author: wf
'''
from skg.wikidata import Wikidata
from skg.dblp import Dblp
from lodstorage.sparql import SPARQL

class Concept:
    """
    an Entity
    """
    def __init__(self,name:str,cls):
        """
        constructor
        
        Args:
            name(str): the name of the node
            cls: a class
        """
        self.name=name
        self.props={}
        self.cls=cls
        if hasattr(cls,"getSamples"):
            for sample in cls.getSamples():
                for key in sample.keys():
                    if not key in self.props:
                        self.props[key]=Property(self,key)
                    
    def map(self,map_name:str,map_list:list):
        """
        map the given list of property mappings under the given map_name
        
        Args:
            map_name(str): the name of the mapping e.g. "wikidata"
            map_list(list): a list of mapping tuples
        """
        for prop_name,mapped_prop in map_list:
            if prop_name in self.props:
                prop=self.props[prop_name]
                prop.setmap(map_name,mapped_prop)
        return self
                    
    def map_wikidata(self,wd_class:str,scholia_suffix,map_list:list):
        """
        map wikidata entries
        
        Args:
            wd_class(str): the main wikidata base class
            scholia_suffix(str): the scholia suffix
        """
        self.wd_class=wd_class
        self.scholia_suffix=scholia_suffix
        self.map("wikidata",map_list)
        return self    
    
            
class Property:
    """
    a Property
    """
    def __init__(self,concept:Concept,name:str):
        """
        constructor
        
        Args:
            concept(Concept): the concept this property belongs to
            name(str): the name of the property
            
        """
        self.concept=concept
        self.name=name
        self.maps={}
        
    def setmap(self,map_name,mapped_prop):
        """
        map the given property
        """
        self.maps[map_name]=mapped_prop
        
    def getmap(self,map_name):
        return self.maps[map_name]
        
    def hasmap(self,map_name:str)->bool:
        """
        check whether there is a mapping for the given map_name
        
        Args:
            map_name(str): the map name to check
            
        Returns:
            bool: True if there is mapping
        """
        return map_name in self.maps
    
class Node:
    """
    a Node in the scholary knowledge graph
    """
    debug=False
    
    def __init__(self):
        """
        constructor
        """
    
    def __str__(self):
        """
        return a text representation of me
        """
        text=f"{self.concept.name} ➞ {self.label}:"
        delim="\n  "
        for prop in self.concept.props.values():
            if hasattr(self, prop.name):
                text+=f"{delim}{prop.name}={getattr(self,prop.name)}"
        return text
        
    def from_dict(self,concept,record:str):
        """
        get my values from the given record
        """
        self.concept=concept
        self.label=record[concept.name]
        for key in concept.props.keys():
            if key in record:
                setattr(self, key, record[key])
                
    def browser_url(self):
        """
        get my browser url
        """
        if self.provenance=="wikidata":
            url=self.scholia_url()
        else:
            url=self.label
        return url
                
    def scholia_url(self):
        """
        get my scholia url
        """
        prefix=f"https://scholia.toolforge.org/{self.concept.scholia_suffix}"
        wd_url=getattr(self, "wikiDataId",None)
        if wd_url is None:
            return None
        else:
            qid=wd_url.replace("http://www.wikidata.org/entity/","")
            return f"{prefix}/{qid}"
        
    @classmethod
    def setProvenance(cls,instances:list,provenance:str):
        """
        set the provenance of the given instances
        """
        for instance in instances:
            instance.provenance=provenance    
        
    @classmethod
    def from_sparql(cls,sparql:SPARQL,sparql_query:str,concept:Concept):
        """
        get instance from the given sparql access point with the given sparql_query for
        the given concept
        
        Args:
            sparql(SPARQL): the sparql access point
            sparql_query(str): the query to execute
            concept(Concept): the concept to create instances for
        """
        if Node.debug:
            print(sparql_query)
        records=sparql.queryAsListOfDicts(sparql_query)
        instances=cls.from_records(records,concept)
        return instances
        
    @classmethod
    def from_records(cls,records:list,concept:Concept):
        """
        get instances from the given records for the given concept
        
        Args:
            records(list): a list of dicts to get instances for
            concept(Concept): the concept to create instances for
        """
        instances=[]
        for record in records:
            instance=cls()
            instance.from_dict(concept,record)
            instances.append(instance)
        return instances
        
    @classmethod
    def from_wikidata_via_id(cls,concept:Concept,id_name:str,id_value:str,lang:str="en"):
        """
        get a node instance from wikidata for the given parameters
        
        Args:
            concept(Concept): the concept to return
            id_name(str): the name of the id to search / lookup with
            id_value(str): the value of the id
            lang(str): the language code to apply
        """
        wikidata=Wikidata()
        if id_name=="wikiDataId":
            value_clause=f"<http://www.wikidata.org/entity/{id_value}>"
        else:
            value_clause=f'''"{id_value}"'''
        sparql_query=f"""# Query for {concept.name} details via ID {id_name} value {id_value}
PREFIX wd: <http://www.wikidata.org/entity/>
PREFIX wdt: <http://www.wikidata.org/prop/direct/>
PREFIX rdfs: <http://www.w3.org/2000/01/rdf-schema#>
SELECT DISTINCT ?{concept.name} ?qId"""
        for prop in concept.props.values():
            sparql_query+=f" ?{prop.name}"
        sparql_query+=f"""
WHERE {{
  VALUES ?{id_name} {{
    {value_clause}
  }}
  # classification!
  ?wikiDataId wdt:P31/wdt:P279* wd:{concept.wd_class}.
  ?wikiDataId rdfs:label ?{concept.name} .
  FILTER(LANG(?{concept.name})="{lang}").
"""
        for prop in concept.props.values():
            if prop.name=="wikiDataId":
                continue
            if not (prop.hasmap("wikidata")):
                raise Exception(f"Property {prop.name} of {concept.name} has no wikidata mapping")
            wd_prop=prop.getmap("wikidata")
            clause=f"?wikiDataId wdt:{wd_prop} ?{prop.name}."
            if prop.name!=id_name:
                clause=f"OPTIONAL {{ {clause} }}"
            sparql_query+="\n  "+clause
        sparql_query+="\n}"
        instances=cls.from_sparql(wikidata.sparql,sparql_query,concept)
        cls.setProvenance(instances, "wikidata")
        return instances
    
    
    @classmethod
    def from_dblp_via_id(cls,concept:Concept,id_name:str,id_value:str,lang:str="en"):
        """
        get a node instance from dblp for the given parameters
        
        Args:
            concept(Concept): the concept to return
            id_name(str): the name of the id to search / lookup with
            id_value(str): the value of the id
            lang(str): the language code to apply
        """
        dblp=Dblp()
        sparql_query=f"""
PREFIX dblp: <https://dblp.org/rdf/schema#>
SELECT 
  ?{concept.name}"""
        for prop in concept.props.values():
            if prop.hasmap("dblp"):
                sparql_query+=f" ?{prop.name}"
        if id_name=="doi":
            value_clause=f"<http://dx.doi.org/{id_value}>"
        elif id_name=="orcid":
            value_clause=f"<https://orcid.org/{id_value}>"
        else:
            value_clause=f'''"{id_value}"'''
        sparql_query+=f"""
WHERE {{
  VALUES ?{id_name} {{
    {value_clause}
  }}
"""
        for prop in concept.props.values():
            if prop.hasmap("dblp"):
                dblp_prop=prop.getmap("dblp")
                sparql_query+=f"""?{concept.name} dblp:{dblp_prop} ?{dblp_prop}.\n"""
        sparql_query+="}\n"
        instances=cls.from_sparql(dblp.sparql,sparql_query,concept)
        cls.setProvenance(instances, "dblp")
        return instances
        


'''
Created on 2022-11-19

@author: wf
'''
class ORCID:
    """
    ORCID handling
    see e.g. https://info.orcid.org/brand-guidelines/#h-orcid-logos-and-icons
    """
    def __init__(self,orcid:str):
        """
        constructor
        
        Args:
            orcid(str): the orcid
        """
        self.orcid=orcid
        
    def asHtml(self,mode:str="full",inline:str="")->str:
        """
        the orcid logo 
        
        Args:
            mode(str): the mode
            inline(str): in inline mode this is the text to be displayed inline 
            
        Returns:
            str: the html code
            
        """
        href=f"""https://orcid.org/{self.orcid}"""
        logo="""<img alt="ORCID logo" src="https://info.orcid.org/wp-content/uploads/2019/11/orcid_16x16.png" width="16" height="16" />"""
        if mode=="full":
            html=f"""<a href="{href}">{logo}{href}</a>"""
        elif mode=="compact":
            html=f"""<a href="{href}">{logo}{self.orcid}</a>"""
        elif mode=="inline":
            html=f"""<a href="{href}">{inline}{logo}</a>"""
        return html


'''
Created on 18.11.2022

@author: wf
'''
from search_engine_parser.core.engines.bing import Search as BingSearch
from search_engine_parser.core.engines.google import Search as GoogleSearch
from search_engine_parser.core.engines.yahoo import Search as YahooSearch
from search_engine_parser.core.engines.duckduckgo import Search as DuckDuckGoSearch
from search_engine_parser.core.engines.googlescholar import Search as GoogleScholarSearch
import sys 
  
class InternetSearch:
    """
    generic internet search
    """
    
    def __init__(self,debug:bool=False):
        """
        constructor
        """
        self.debug=debug
        self.gsearch = GoogleSearch()
        self.ysearch = YahooSearch()
        self.bsearch = BingSearch()
        self.dsearch = DuckDuckGoSearch()
        self.gs_search=GoogleScholarSearch()
        self.engines=[self.gs_search,self.ysearch,self.dsearch,self.bsearch]
        
    def handleException(self,ex):
        """
        handle the given exception
        """
        if self.debug:
            print(f"{str(ex)}",file=sys.stderr)
        
    def search(self,search_term:str):
        """
        search my engines for the given search_term
        """
        search_args=(search_term, 1)
        for engine in self.engines:
            try:
                result=engine.search(*search_args)
                yield engine.name,result.results
                pass
            except Exception as ex:
                self.handleException(ex)
                pass
            
    


'''
Created on 2022-11-21

@author: wf
'''
from tests.base_skg_test import BaseSkgTest
from skg.graph import Node
from skg.location import Country

class TestLocation(BaseSkgTest):
    """
    test concerning the scholar/author concept
    """
    
    def test_locations_by_wikidata_id(self):
        """
        test searching locations 
        """
        country_concept=self.skg_def.concepts["Country"]
        id_examples=[
            {
                "id_name": "iso_code",
                "id_value": "SG",
                "concept": country_concept,
                "example": Country.getSamples()[0]
            },
        ]
        
        def checkItem(item:Node,id_name:str,id_value:str,debug:bool=False):
            """
            check the given item
            
            Args:
                item(Node): the item to check
                id_name(str): the name of the id used to retrieve the item
                id_value(str) the value that has been used to retriebe the item
                debug(bool): if True show debug information
            """
            # @TODO check against id_example
            self.assertEqual("SG",item.iso_code)
            self.assertEqual(5866139.0,item.population)
            self.assertTrue("Q334" in item.wikiDataId)
            pass
            
            
        debug=self.debug
        debug=True
        self.check_id_examples(id_examples, createFunc=Node.from_wikidata_via_id,checkItem=checkItem,debug=debug)            
